<?php

declare(strict_types=1);

namespace Drupal\key_per_user\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\field_encrypt\Plugin\Field\FieldType\EncryptedFieldStorageItem;
use Drupal\key_per_user\Exception\KeyPerUserException;
use Drupal\key_per_user\KeyPerUserInterface;
use Drupal\user\Entity\User;
use Webmozart\Assert\Assert;

/**
 * Plugin implementation of the 'encrypted_field_storage' field type.
 *
 * Customized to use a different key for each user.
 */
#[FieldType(
  id: "key_per_user_encrypted_field_storage",
  label: new TranslatableMarkup("Key per user encrypted field storage"),
  description: new TranslatableMarkup("Stores encrypted field data."),
  no_ui: TRUE,
)]
class KeyPerUserEncryptedFieldStorageItem extends EncryptedFieldStorageItem {

  /**
   * Checks whether a bundle should be encrypted on a per-user basis.
   *
   * @param string $entity_bundle
   *   The bundle to check.
   *
   * @return bool
   *   TRUE if the bundle should be encrypted separately for each user.
   */
  public static function isEncryptedSeparatelyPerUser(string $entity_bundle): bool {
    /** @var \Drupal\Core\Session\AccountProxyInterface $current_user */
    $current_user = \Drupal::service('current_user');
    $user_id = $current_user->id();

    // Do nothing for the anonymous user.
    if ($user_id == 0) {
      return FALSE;
    }

    $account = User::load($user_id);
    if ($account instanceof KeyPerUserInterface) {
      $encrypted_per_user_bundles = $account->getEncryptedPerUserBundles();
      return (in_array($entity_bundle, $encrypted_per_user_bundles, TRUE));
    }
    throw new KeyPerUserException('Accounts must implement KeyPerUserInterface.');
  }

  /**
   * Encrypts the field item.
   */
  public function encrypt(): void {
    // If the decrypted value is set, encrypt it and overwrite.
    if ($this->decrypted_value !== NULL) {
      $encryption_profile = \Drupal::config('field_encrypt.settings')->get('encryption_profile');
      if (!is_string($encryption_profile)) {
        throw new KeyPerUserException('Encryption profile in config must be a string.');
      }

      $entity_bundle = $this->getEntity()->bundle();
      // For users, use the configured encryption profile.
      if (self::isEncryptedSeparatelyPerUser($entity_bundle)) {
        $encryption_profile = 'key_per_user';
      }

      $this->value = base64_encode(
        \Drupal::service('encryption')->encrypt(
          serialize($this->decrypted_value),
          $this->loadEncryptionProfile($encryption_profile)
        )
      );

      if ($this->get('encryption_profile')->getValue() !== $encryption_profile) {
        $this->encryption_profile = $encryption_profile;
      }
    }
    else {
      $this->value = NULL;
    }
    $this->decrypted_value = NULL;
  }

  /**
   * Decrypts the field item.
   */
  public function decrypt(): ?array {
    if ($this->value !== NULL) {
      // Use the field's encryption_profile if available. It should always be.
      $encryption_profile = $this->get('encryption_profile')->getValue();
      Assert::stringNotEmpty($encryption_profile, 'Failed to get field encryption profile for decrypting! %s');
      $base64_decoded_value = base64_decode($this->value, TRUE);
      Assert::stringNotEmpty($base64_decoded_value, 'Failed to base64 decode the value! %s');
      try {
        $decrypted_result = unserialize(
          \Drupal::service('encryption')->decrypt(
            $base64_decoded_value,
            $this->loadEncryptionProfile($encryption_profile)
          ),
          ['allowed_classes' => FALSE]
        );
        Assert::isArray($decrypted_result, 'Decrypted result was not an array! %s');
        return $decrypted_result;
      }
      // @phpstan-ignore-next-line: caught "Exception" must be rethrown
      catch (\Exception $e) {
        $message = $e->getMessage();
        \Drupal::logger('key_per_user')->error("Failed to decrypt: $message");
      }
    }
    return NULL;
  }

}
