<?php

declare(strict_types=1);

namespace Drupal\key_per_user\Plugin\KeyProvider;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyProviderBase;
use Drupal\key_per_user\Exception\KeyPerUserException;
use Drupal\key_per_user\KeyPerUserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A key provider that allows a key to be stored in the user account.
 *
 * @KeyProvider(
 *   id = "per_user_key",
 *   label = @Translation("Per-user key"),
 *   description = @Translation("The per-user key provider enables a key that is retrieved from the user account. You must add custom code to provide the key."),
 *   storage_method = "env",
 *   key_value = {
 *     "accepted" = FALSE,
 *     "required" = FALSE
 *   }
 * )
 */
class KeyPerUserProvider extends KeyProviderBase implements KeyPluginFormInterface, ContainerFactoryPluginInterface {

  public function __construct(
    array $configuration,
    string $plugin_id,
    mixed $plugin_definition,
    protected AccountInterface $currentUser,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'base64_encoded' => FALSE,
      'strip_line_breaks' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['strip_line_breaks'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Strip trailing line breaks'),
      '#description' => $this->t('Check this to remove any trailing line breaks from the variable. Leave unchecked if there is a chance that a line break could be a valid character in the key.'),
      '#default_value' => $this->getConfiguration()['strip_line_breaks'],
    ];

    // If this key type is for an encryption key.
    $form_object = $form_state->getFormObject();
    if (!method_exists($form_object, 'getEntity')) {
      throw new KeyPerUserException('Invalid form class! Does not have getFormObject method!');
    }
    if ($form_object->getEntity()->getKeyType()->getPluginDefinition()['group'] == 'encryption') {
      // Add an option to indicate that the value is stored Base64-encoded.
      $form['base64_encoded'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Base64-encoded'),
        '#description' => $this->t('Check this if the key in the variable is Base64-encoded.'),
        '#default_value' => $this->getConfiguration()['base64_encoded'],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $key_value = $this->getKeyFromUserAccount();

    // Is the key set for the current user?
    if ($key_value == '') {
      // Use 'base64_encoded' because there is no suitable field for the user
      // key, since it is not exposed configuration.
      $form_state->setErrorByName('base64_encoded', $this->t('The user account variable could not be read.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyValue(KeyInterface $key): string {
    $key_value = $this->getKeyFromUserAccount();

    if (isset($this->configuration['strip_line_breaks']) && $this->configuration['strip_line_breaks']) {
      $key_value = rtrim($key_value, "\n\r");
    }

    if (isset($this->configuration['base64_encoded']) && $this->configuration['base64_encoded']) {
      $key_value = base64_decode($key_value, TRUE);
      if ($key_value === FALSE) {
        throw new KeyPerUserException('Failed to base64 decode the key value!');
      }
    }

    return $key_value;
  }

  /**
   * Gets the encryption key value from the user account.
   *
   * @return string
   *   The key value from the user account as text.
   */
  private function getKeyFromUserAccount(): string {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $user_id = $this->currentUser->id();

    // If the user is anonymous, return an empty string.
    if ($user_id == 0) {
      return '';
    }

    $account = $user_storage->load($user_id);
    if ($account instanceof KeyPerUserInterface) {
      return $account->getEncryptionKey();
    }
    throw new KeyPerUserException('Accounts must implement KeyPerUserInterface.');
  }

}
